import os
import logging

import zope.interface
from acme import challenges
from certbot import interfaces
from certbot.plugins import common
from redis import StrictRedis
from .redisurlparser import RedisUrlParser


logger = logging.getLogger(__name__)

@zope.interface.implementer(interfaces.IAuthenticator)
@zope.interface.provider(interfaces.IPluginFactory)
class Authenticator(common.Plugin):

    description = "Redis Authenticator"
    @classmethod
    def add_parser_arguments(cls, add):
        add("redis-url", default=os.getenv('REDIS_URL'),
            help="redis-url url of the redis insteance you want to store your challenges in")

    def __init__(self, *args, **kwargs):
        super(Authenticator, self).__init__(*args, **kwargs)
        self._httpd = None
        self.redis_client = StrictRedis.from_url(self.conf('redis-url'), socket_keepalive=True)

    def prepare(self):  # pylint: disable=missing-docstring,no-self-use
        pass  # pragma: no cover

    def more_info(self):  # pylint: disable=missing-docstring,no-self-use
        return ("")

    def get_chall_pref(self, domain):
        # pylint: disable=missing-docstring,no-self-use,unused-argument
        return [challenges.HTTP01]

    def _get_key(self, achall):   # pylint: disable=missing-docstring
        key = achall.chall.path[1:].split("/")[2]
        return str(key)

    def perform(self, achalls):  # pylint: disable=missing-docstring
        responses = []
        for achall in achalls:
            responses.append(self._perform_single(achall))
        return responses

    def _perform_single(self, achall):
        # upload the challenge file to redis
        # then run simple http verification
        response, validation = achall.response_and_validation()

        self.redis_client.setex(self._get_key(achall), 60, validation)
        if response.simple_verify(
                achall.chall, achall.domain,
                achall.account_key.public_key(), self.config.http01_port):
            return response
        else:
            logger.error(
                "Self-verify of challenge failed, authorization abandoned!")
            return None

    def cleanup(self, achalls):
        # pylint: disable=missing-docstring,no-self-use,unused-argument
        return None



try:
    from urllib import parse
except ImportError:
    import urlparse as parse

class RedisUrlParser:
    def __init__(self, redis_url):
        if 'redis' not in parse.uses_netloc:
            parse.uses_netloc.append('redis')

        url = parse.urlparse(redis_url)  # type: parse.ParseResult
        self._hostname = url.hostname
        self._port = url.port
        self._password = url.password
        self._key = url.path.strip('/')


    @property
    def hostname(self):
        return self._hostname


    @property
    def port(self):
        return self._port


    @property
    def password(self):
        return self._password

    @property
    def key(self):
        return self._key



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))



setup(
    name='certbot-redis',  # Required
    version='0.1.8',  # Required
    description='Certbot plugin for Redis',
    url='https://github.com/deathowl/certbot-redis-plugin',  # Optional

    author='Balint Csergo',  # Optional

    author_email='<bcsergo@emarsys.com>',  # Optional

    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='certbot redis',  # Optional

    packages=find_packages(),  # Required

    install_requires=[
        'acme>=0.22.0',
        'certbot>=0.22.0',
        'PyOpenSSL',
        'setuptools',
        'zope.component',
        'zope.event',
        'zope.interface',
        'hiredis',
        'redis',
    ],
    include_package_data=True,
    entry_points={
        'letsencrypt.plugins': [
            'redis = certbot_redis.plugin:Authenticator',
        ],
    },



)
